# - *- coding: utf- 8 - *-

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.Standby import TryQuitMainloop

from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigYesNo, ConfigInteger, ConfigDescription, fileExists
from Components.ConfigList import ConfigList, ConfigListScreen

#============ Version =============
version = "V1.3"

#============ config =============
config.plugins.TimeshiftPlus = ConfigSubsection()
config.plugins.TimeshiftPlus.active = ConfigYesNo(default=True)
config.plugins.TimeshiftPlus.jumpActive = ConfigYesNo(default=True)
config.plugins.TimeshiftPlus.jumpWidthStep = ConfigInteger(default = 30, limits = (10, 999))
config.plugins.TimeshiftPlus.jumpWidthRepeat = ConfigInteger(default = 240, limits = (10, 999))
config.plugins.TimeshiftPlus.jumpRepeatInterval = ConfigInteger(default = 600, limits = (100, 3000))
config.plugins.TimeshiftPlus.preJumpCount = ConfigSelection(default=False, choices=[(False,"---"),("1","1"),("2","2"),("3","3"),("4","4"),("5","5"),("6","6")])
config.plugins.TimeshiftPlus.limitJumpRight = ConfigInteger(default = 0, limits = (0, 99))
config.plugins.TimeshiftPlus.stopWithExit = ConfigSelection(default=False, choices=[(False,"---"),("LiveTV","LiveTV"),("LiveTVIBOff","LiveTV (nur bei IB off)"),("LiveTVPause","LiveTV (nur bei TS-Pause)"),("LiveTVPauseIBOff","LiveTV (nur bei TS~Pause und IB off)")])
config.plugins.TimeshiftPlus.tsPauseIBOff = ConfigYesNo(default=False)
config.plugins.TimeshiftPlus.showStateAfterJump = ConfigYesNo(default=True)



#============ TimeshiftPlusConfigScreen =============
class TimeshiftPlusConfigScreen(Screen, ConfigListScreen):
	def __init__(self, session):
		Screen.__init__(self, session)		
		self.skinName = [ "TimeshiftPlusConfig", "Setup" ]
		self.setTitle(_("TimeshiftPlus - Einstellungen") + " " + version)
		
		self.list = []
		self.buildConfigList()
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changedEntry)
		
		self["config"].onSelectionChanged.append(self.updateDesc)
		
		self["actions"] = ActionMap(["ColorActions", "SetupActions"],
			{
				"cancel": self.keyCancel,
				"save": self.keyOK,
				"red": self.keyCancel,
				"green": self.keyGreen,
			}, -2)

		self["key_green"] = StaticText(_("Save"))
		self["key_red"] = StaticText(_("Cancel"))

	def buildConfigList(self):
		self.list = []
		
		self.list.append(getConfigListEntry(_("TimeshiftPlus aktivieren"), config.plugins.TimeshiftPlus.active, ""))
		
		if config.plugins.TimeshiftPlus.active.value:
		
			self.list.append(getConfigListEntry(_("Bedienung mit Steuerkreuz Links/Rechts aktivieren"), config.plugins.TimeshiftPlus.jumpActive,_("Steuerkreuz-Links: TS aktivieren und springen (TS muss vorher vorher durch autom.TS, Pause, Play, Rewind, Cursor-Right... gestartet worden sein) / Steuerkreuz-Rechts TS starten und Pause")))
			
			if config.plugins.TimeshiftPlus.jumpActive.value:
			
				self.list.append(getConfigListEntry(_("Sprungweite Links/Rechts (Drücken) [s]"), config.plugins.TimeshiftPlus.jumpWidthStep, _("Sprungweite beim Drücken von Steuerkreuz Links/Rechts")))
				
				self.list.append(getConfigListEntry(_("Sprungweite Links/Rechts (Drücken und Halten) [Repeat] [s]"), config.plugins.TimeshiftPlus.jumpWidthRepeat, _("Sprungweite beim Drücken und Halten von Steuerkreuz Links/Rechts")))
				
				self.list.append(getConfigListEntry(_("Sprung-Repeat-Intervall [ms]"), config.plugins.TimeshiftPlus.jumpRepeatInterval, _("Wartezeit zwischen Sprüngen, beim Drücken und Halten von Steuerkreuz Links/Rechts [Default: 600ms]")))
				
				self.list.append(getConfigListEntry(_("Anzahl vorgelagerter 10s-Sprünge [Drücken]"), config.plugins.TimeshiftPlus.preJumpCount, _("Anzahl vorgelagerter 10s-Sprünge, vor Sprüngen mit eingestellter Sprungweite beim Drücken. Die vorgelagerten Sprünge werden ausgeführt beim Start, Richtungswechsel oder nach 3s ohne Sprung")))
				
				self.list.append(getConfigListEntry(_("Sprung-Rechts vor dem Ende begrenzen (0 = aus) [s]"), config.plugins.TimeshiftPlus.limitJumpRight, _("Sprünge werden bis zur eingest. Zeit vor dem TS-Ende begrenzt, um das Verlassen von Timeshift zu verhindern (wird ein Sprung-Rechts bei -aus- dennoch begrenzt, stammt die Begrenzung von VTi->Aufnahmen->Vorspulen am Ende des Films stoppen)")))
				
				self.list.append(getConfigListEntry(_("Aktion beim Drücken der EXIT-Taste"), config.plugins.TimeshiftPlus.stopWithExit, _("Timeshift sofort beenden und LiveTV anzeigen  (IB off = Infobar ausgeblendet, TS-Pause = Timeshift-Status-Pause)")))
				
			self.list.append(getConfigListEntry(_("Infobar bei TS-Pause autom. ausblenden"), config.plugins.TimeshiftPlus.tsPauseIBOff, _("Infobar nicht nur bei TS-Play, sondern auch bei TS-Pause autom. ausblenden (Infoleiste beim Spulen anzeigen aktiviert)")))
			
			self.list.append(getConfigListEntry(_("Nach Sprung aktuellen Zustand anzeigen"), config.plugins.TimeshiftPlus.showStateAfterJump, _("Nach Sprung, aktuellen Zustand in der Infobar zeitverzögert wieder anzeigen (Play, Pause, ...)")))
		
	def changedEntry(self):
		self.buildConfigList()
		self["config"].setList(self.list)
	
	def updateDesc(self):
		cur = self["config"].getCurrent()
		if cur and len(cur)>2:
			self["configdesc"].text = cur[2]
	
	def keyOK(self):
		return
		#self.keyGreen()

	def keyGreen(self):
		for x in self["config"].list:
			x[1].save()
		#Restart GUI
		msg = self.session.openWithCallback(self.restartGUI,MessageBox,_("Restart necessary, restart GUI now?"), MessageBox.TYPE_YESNO)
		msg.setTitle(_("Message"))
		
	def restartGUI(self,answer):		
		if answer:
			self.session.open(TryQuitMainloop, 3)
		else:
			self.close()
			
			
		