# - *- coding: utf- 8 - *-
from Plugins.Plugin import PluginDescriptor
from Screens.InfoBarGenerics import InfoBarSeek, InfoBarTimeshiftState
from Screens.InfoBar import InfoBar
from Screens.MessageBox import MessageBox
from Components.ActionMap import ActionMap
from Components.config import config
from functools import partial
from enigma import eTimer, iPlayableService
from Components.ServiceEventTracker import ServiceEventTracker
import time
import os

from .TimeshiftPlusConfig import TimeshiftPlusConfigScreen


jumpWidthStep = 30 #s
jumpWidthRepeat = 120 #s
jumpRepeatInterval = 600 #ms
jumpPosMin = 90000 # 1s
jumpPosMax = 1800000 #20s
lockKeysIB = False
lockKeysIBTimer = eTimer()
lockKeysTS = False
lockKeysTSTimer = eTimer()
keyLeftRepeatTimer = eTimer()
keyRightRepeatTimer = eTimer()
keyLeftRepeatWasActive = False
keyRightRepeatWasActive = False
activateJumpDelayTimer = eTimer()
preJumpActive = False
preJumpCount = 2
preJumpLastDir = ""
preJumpResetTimer = eTimer()
wasIBShown = False
blockIBShow = False

orgInfoBarSeekInit = None
orgSetSeekStateText = None
orgClearSeekStateText = None
orgCheckSkipShowHideLock = None


logFile = "/tmp/TimeshiftPlusLog.log"

def writeLog(logText):
	t = time.localtime()
	logTime = time.strftime("%d.%m.%Y %H:%M:%S: ",t)
	log = open(logFile,"a")
	log.write(str(logTime) + str(logText) + "\n")
	log.close()

def main(session, **kwargs):
	session.open(TimeshiftPlusConfigScreen)
	
def sessionstart(reason, **kwargs):
	if reason == 0:
		
		#========== sessionstart =============
		if not config.plugins.TimeshiftPlus.active.value:
			return None
			
		print "Init-TimeshiftPlus"
		
		#Jump Left/Right
		if config.plugins.TimeshiftPlus.jumpActive.value:
			#InfoBarSeek
			global orgInfoBarSeekInit
			orgInfoBarSeekInit = InfoBarSeek.__init__
			InfoBarSeek.__init__ = newInfoBarSeekInit
		
			#JumpWidth/JumpPosMax(0=deactivated)
			global jumpWidthStep, jumpWidthRepeat, jumpRepeatInterval, jumpPosMax 
			jumpWidthStep = int(config.plugins.TimeshiftPlus.jumpWidthStep.value)
			jumpWidthRepeat = int(config.plugins.TimeshiftPlus.jumpWidthRepeat.value)
			jumpRepeatInterval = int(config.plugins.TimeshiftPlus.jumpRepeatInterval.value)
			jumpPosMax = int(config.plugins.TimeshiftPlus.limitJumpRight.value) * 90000
			
			#preJump
			global preJumpActive, preJumpCount
			if config.plugins.TimeshiftPlus.preJumpCount.value:
				preJumpActive = True
				preJumpCount = int(config.plugins.TimeshiftPlus.preJumpCount.value)
				preJumpResetTimer.callback.append(onPreJumpResetTimer)
			
			#lockKeyTimer
			lockKeysIBTimer.callback.append(onLockKeysIBTimer)
			lockKeysTSTimer.callback.append(onLockKeysTSTimer)
			
		#Hide IB also at Pause
		if config.plugins.TimeshiftPlus.tsPauseIBOff.value:
			global orgCheckSkipShowHideLock
			orgCheckSkipShowHideLock = InfoBarSeek.checkSkipShowHideLock
			InfoBarSeek.checkSkipShowHideLock = newCheckSkipShowHideLock
			
			
		#Fix TimeshiftState show/hide
		InfoBarTimeshiftState._InfoBarPVRState__playStateChanged = newTimeshiftPlayStateChanged #SeekState changed
		InfoBarTimeshiftState._mayShow = newTimeshiftMayShow
		global orgSetSeekStateText #change text without SeekState change
		orgSetSeekStateText = InfoBarSeek.setSeekStateText
		InfoBarSeek.setSeekStateText = newSetSeekStateText
		global orgClearSeekStateText #callback SeekTimer
		orgClearSeekStateText = InfoBarSeek.clearSeekStateText
		InfoBarSeek.clearSeekStateText = newClearSeekStateText

#======= activateTimeshiftAndJump (keyLeft_IB) ======
def activateTimeshiftAndJump(self):
	
	ts = self.getTimeshift()
	if ts is None:
		self.session.open(MessageBox, _("Timeshift not possible!"), MessageBox.TYPE_ERROR, timeout = 2)
		return 0
		
	if not self.timeshift_enabled:
		return 0
	
	#activate Timeshift
	if not ts.isTimeshiftActive():
		ts.activateTimeshift()
		
		print "TimeshiftPlus - Timeshift activated"
		
		#Prevent Jump after start/activation
		global lockKeysTS
		lockKeysTS = True
		lockKeysTSTimer.stop()
		lockKeysTSTimer.start(1000, True)
		global lockKeysIB
		lockKeysIB = True
		lockKeysIBTimer.stop()
		lockKeysIBTimer.start(1000, True)
		
		#play and jumpLeft
		self.setSeekState(self.SEEK_STATE_PLAY)
		activateJumpDelayTimer.start(300,True)
		
#======= startTimeshiftAndPause (keyRight_IB) ======
def startTimeshiftAndPause(self):
	
	ts = self.getTimeshift()
	if ts is None:
		self.session.open(MessageBox, _("Timeshift not possible!"), MessageBox.TYPE_ERROR, timeout = 2)
		return 0
		
	if self.timeshift_enabled:
		self.activateTimeshiftEndAndPause()
	else:
		self.startTimeshift()
		
	#Prevent Jump after start/activation
	global lockKeysTS
	lockKeysTS = True
	lockKeysTSTimer.stop()
	lockKeysTSTimer.start(1000, True)
	global lockKeysIB
	lockKeysIB = True
	lockKeysIBTimer.stop()
	lockKeysIBTimer.start(1000, True)
		
	print "TimeshiftPlus - Timeshift started"
	
#========= jumpLeft ==========
def jumpLeft(self, jumpWidth):
	
	seek = self.getSeek()
	
	if seek:
		#ensure play before jump
		if self.seekstate != self.SEEK_STATE_PLAY:
			self.unPauseService()
			
		#find JumpPos
		pos = seek.getPlayPosition()[1]
		jumpDelta = pos - jumpPosMin
		if jumpDelta > (jumpWidth*90000):
			jumpDelta = jumpWidth*90000
			
		#execute Jump
		if jumpDelta > 180000:
			jumpLabel = -1 * int(jumpDelta / 90000)
			print "TimeshiftPlus - JumpLeft rel. from pos: " + str(-1 * jumpDelta)
			self.doSeekRelative(-1 * jumpDelta)
		else:
			jumpLabel = 0
		
		#show TimeshiftState/IB
		self.setSeekStateText(jumpLabel,True)
		if config.usage.show_infobar_on_skip.value:
			self.showAfterSeek()

#========= jumpRight ==========
def jumpRight(self, jumpWidth):
	
	seek = self.getSeek()
		
	if seek:
		#ensure play before jump
		if self.seekstate != self.SEEK_STATE_PLAY:
			self.unPauseService()
		
		#find JumpPos
		if jumpPosMax > 0:
			pos = seek.getPlayPosition()[1]
			end = seek.getLength()[1]
			jumpDelta = end - jumpPosMax - pos
			if jumpDelta > (jumpWidth*90000):
				jumpDelta = jumpWidth*90000
		else:
			jumpDelta = jumpWidth*90000
		
		#execute Jump
		if jumpDelta > 180000:
			jumpLabel = int(jumpDelta / 90000)
			print "TimeshiftPlus - JumpRight rel. from pos: " + str(jumpDelta)
			self.doSeekRelative(jumpDelta)
		else:
			jumpLabel = 0

		#show TimeshiftState/IB
		self.setSeekStateText(jumpLabel)
		if config.usage.show_infobar_on_skip.value:
			self.showAfterSeek() #IB
		
#========= keyActions ==========
def keyLeft_down_TS(self):
	if isInfoBar(self):
		#Repeat delay
		keyLeftRepeatTimer.start(800,True)
	else:
		self.seekBackLeft() #org
	return 1
	
def keyLeft_up_TS(self):
	global keyLeftRepeatWasActive, preJumpCount, preJumpLastDir
	keyLeftRepeatTimer.stop()
	if isInfoBar(self):
		if not lockKeysTS and not keyLeftRepeatWasActive:
			if preJumpActive:
				preJumpResetTimer.stop()
				preJumpResetTimer.start(3000,True)
				if preJumpLastDir != "Left":
					preJumpResetCounter()
				preJumpLastDir = "Left"
				if preJumpCount > 0:
					jumpLeft(self,10)
					preJumpCount = preJumpCount - 1
				else:
					jumpLeft(self,jumpWidthStep)
			else:
				jumpLeft(self,jumpWidthStep)
			
	keyLeftRepeatWasActive = False
	return 1
	
def onKeyLeftRepeatTimer(self):
	global keyLeftRepeatWasActive
	keyLeftRepeatWasActive = True
	jumpLeft(self,jumpWidthRepeat)
	keyLeftRepeatTimer.start(jumpRepeatInterval,True)
	
def keyRight_down_TS(self):
	if isInfoBar(self):
		#Repeat delay
		keyRightRepeatTimer.start(800,True)
	else:
		self.seekFwdRight() #org
	return 1
	
def keyRight_up_TS(self):
	global keyRightRepeatWasActive, preJumpCount, preJumpLastDir
	keyRightRepeatTimer.stop()
	if isInfoBar(self):
		if not lockKeysTS and not keyRightRepeatWasActive:
			if preJumpActive:
				preJumpResetTimer.stop()
				preJumpResetTimer.start(3000,True)
				if preJumpLastDir != "Right":
					preJumpResetCounter()
				preJumpLastDir = "Right"
				if preJumpCount > 0:
					jumpRight(self,10)
					preJumpCount = preJumpCount - 1
				else:
					jumpRight(self,jumpWidthStep)
			else:
				jumpRight(self,jumpWidthStep)
	keyRightRepeatWasActive = False
	return 1
	
def onKeyRightRepeatTimer(self):
	global keyRightRepeatWasActive
	keyRightRepeatWasActive = True
	jumpRight(self,jumpWidthRepeat)
	keyRightRepeatTimer.start(jumpRepeatInterval,True)
	
def keyExit_down_TS(self):
	global wasIBShown
	if self.shown:
		wasIBShown = True
	else:
		wasIBShown = False
	return 0
	
def keyExit_up_TS(self):
	global blockIBShow
	showLiveTV = False
	ts = self.getTimeshift()
	if not ts is None and ts.isTimeshiftActive():
		if config.plugins.TimeshiftPlus.stopWithExit.value == "LiveTV":
			showLiveTV = True
		elif config.plugins.TimeshiftPlus.stopWithExit.value == "LiveTVIBOff" and not wasIBShown:
			showLiveTV = True
		elif config.plugins.TimeshiftPlus.stopWithExit.value == "LiveTVPause" and self.seekstate == self.SEEK_STATE_PAUSE:
			showLiveTV = True
		elif config.plugins.TimeshiftPlus.stopWithExit.value == "LiveTVPauseIBOff" and self.seekstate == self.SEEK_STATE_PAUSE and not wasIBShown:
			showLiveTV = True
			
	if showLiveTV:
		#switch to Live-TV (TS is still running in the background)
		blockIBShow = True
		self.stopAutoTimeShift()
		return 1
	else:
		return 0

	
def keyLeft_IB(self):
	if isInfoBar(self) and not lockKeysIB:
		activateTimeshiftAndJump(self)
	return 1
		
def keyRight_IB(self):
	if isInfoBar(self) and not lockKeysIB:
		startTimeshiftAndPause(self)
	return 1
		
def keyDummy():
	return 1
		
def onLockKeysIBTimer():
	global lockKeysIB
	lockKeysIB = False
	
def onLockKeysTSTimer():
	global lockKeysTS
	lockKeysTS = False
	
def onActivateJumpDelayTimer(self):
	if int(config.plugins.TimeshiftPlus.limitJumpRight.value):
		jumpLeft(self,jumpWidthStep)
	else:
		jumpLeft(self,jumpWidthStep-5)
	
def onPreJumpResetTimer():
	preJumpResetCounter()
	
def preJumpResetCounter():
	global preJumpCount
	preJumpCount = int(config.plugins.TimeshiftPlus.preJumpCount.value)

def onTimeshiftSeekableStatusChanged(self):
	global keyLeftRepeatWasActive, keyRightRepeatWasActive, lockKeysIB
	
	#Reset Repeat-Timer after Jump out End
	keyLeftRepeatWasActive = False
	keyRightRepeatWasActive = False
	keyLeftRepeatTimer.stop()
	keyRightRepeatTimer.stop()
	
	#Prevent IB-KeyActions direct after Jump out End
	if self.timeshift_enabled:
		ts = self.getTimeshift()
		if not ts is None and not ts.isTimeshiftActive():
			lockKeysIB = True
			lockKeysIBTimer.stop()
			lockKeysIBTimer.start(1000, True)

#============= newInfoBarSeekInit =============
def newInfoBarSeekInit(self, actionmap = "InfobarSeekActions"):
	orgInfoBarSeekInit(self, actionmap)
	
	#Infobar
	if isInfoBar(self) and actionmap == "InfobarSeekActions":
		
		if config.plugins.TimeshiftPlus.stopWithExit.value:
			self["SeekActionsCross"] = ActionMap(["TimeshiftPlusJump"],
			{
				"key_left_down": partial(keyLeft_down_TS,self),
				"key_left_up": partial(keyLeft_up_TS,self),
				"key_right_down": partial(keyRight_down_TS,self),
				"key_right_up": partial(keyRight_up_TS,self),
				"key_exit_down": partial(keyExit_down_TS,self),
				"key_exit_up": partial(keyExit_up_TS,self),
			},-1)
		else:
			self["SeekActionsCross"] = ActionMap(["TimeshiftPlusJump"],
			{
				"key_left_down": partial(keyLeft_down_TS,self),
				"key_left_up": partial(keyLeft_up_TS,self),
				"key_right_down": partial(keyRight_down_TS,self),
				"key_right_up": partial(keyRight_up_TS,self),
			},-1)
		
		self["TimeshiftPlusActivateActions"] = ActionMap(["TimeshiftPlusActivate"],
		{
			"key_left": partial(keyLeft_IB,self),
			"key_right": partial(keyRight_IB,self),
			"key_dummy": keyDummy,
		},0)
		
		del keyLeftRepeatTimer.callback[:]
		keyLeftRepeatTimer.callback.append(partial(onKeyLeftRepeatTimer,self))
		del keyRightRepeatTimer.callback[:]
		keyRightRepeatTimer.callback.append(partial(onKeyRightRepeatTimer,self))
		del activateJumpDelayTimer.callback[:]
		activateJumpDelayTimer.callback.append(partial(onActivateJumpDelayTimer,self))

		self.my_event_tracker = ServiceEventTracker(screen=self, eventmap=
		{
			iPlayableService.evSeekableStatusChanged: partial(onTimeshiftSeekableStatusChanged,self)
		})

#===== newTimeshiftPlayStateChanged (seekstate changed) =======
def newTimeshiftPlayStateChanged(self,state):
	global blockIBShow
	self.SeekStateTimer.stop()
	self._InfoBarTimeshiftState__hideTimer.stop()
	
	seekstateStr = state[3]
	self.pvrStateDialog["state"].setText(seekstateStr)
	
	if (not config.usage.show_infobar_on_skip.value and self.seekstate == self.SEEK_STATE_PLAY and not self.force_show) or not self.timeshift_enabled:
		self.pvrStateDialog.hide()
	else:
		if config.usage.show_infobar_on_skip.value and self.execing and self.timeshift_enabled and not self.shown:
			if not blockIBShow:
				self.doShow() #IB
		self._mayShow()		
	
	blockIBShow = False

#===== newTimeshiftMayShow (called also from IB show event) =======
def newTimeshiftMayShow(self):
	self._InfoBarTimeshiftState__hideTimer.stop()
	if self.execing and self.timeshift_enabled:
		if self.shown:
			ts = self.getTimeshift()
			if ts and ts.isTimeshiftActive():
				self.pvrStateDialog.show()
		else:
			self.pvrStateDialog.show()
			if not config.usage.show_infobar_on_skip.value:
				#option not wind or slowMotiom
				#if not (self.seekstate[1] != 0 or self.seekstate[2] != 0):
				#not wind
				if not self.seekstate[1] != 0:
					#display duration: Play/Pause/SloMo
					self._InfoBarTimeshiftState__hideTimer.start(3000,True)

#===== newSetSeekStateText (only Text without seekstate change) ======
def newSetSeekStateText(self, seekval, zeroNegative = False):
	if not isInfoBar(self):
		orgSetSeekStateText(self,seekval)
		return None
	
	self._InfoBarTimeshiftState__hideTimer.stop()
	self.SeekStateTimer.stop()
	seektext = ""
	if seekval < 0 or (seekval == 0 and zeroNegative):
		seektext = "- "
	else:
		seektext = "+ "
	seektext = seektext + str(abs(seekval)) + " s"
	self.pvrStateDialog["state"].setText(seektext)
	self.pvrStateDialog.show()
	#display duration: Jump-State
	self.SeekStateTimer.start(2000, True)

#===== newClearSeekStateText (callback SeekStateTimer) ======
def newClearSeekStateText(self):
	if not isInfoBar(self):
		orgClearSeekStateText(self)
		return None
		
	if config.plugins.TimeshiftPlus.showStateAfterJump.value:
		seekstateStr = self.seekstate[3]
		self.pvrStateDialog["state"].setText(seekstateStr)
	
	if not self.shown: #and not self.seekstate[0] if Pause shoud not be hide
		if config.plugins.TimeshiftPlus.showStateAfterJump.value:
			#display duration: option show Play after Jump instead direct hide
			#self._InfoBarTimeshiftState__hideTimer.start(1500,True)
			self.pvrStateDialog.hide()
		else:
			self.pvrStateDialog.hide()

#===== newCheckSkipShowHideLock ======
def newCheckSkipShowHideLock(self):
	if not isInfoBar(self):
		orgCheckSkipShowHideLock(self)
		return None
	
	#wantlock = self.seekstate != self.SEEK_STATE_PLAY 
	wantlock = self.seekstate != self.SEEK_STATE_PLAY and self.seekstate[3] != "||"
	
	if config.usage.show_infobar_on_skip.value:
		if self.lockedBecauseOfSkipping and not wantlock:
			self.unlockShow()
			self.lockedBecauseOfSkipping = False
		#======= restart hideTimer ======
		elif not self.lockedBecauseOfSkipping and not wantlock:
			self.lockShow()
			self.unlockShow()
		# ===============================
		if wantlock and not self.lockedBecauseOfSkipping:
			self.lockShow()
			self.lockedBecauseOfSkipping = True
			

#===== isInfoBar ======
def isInfoBar(self):
	return self.__class__.__name__ == "InfoBar"

	
#========== Plugin-Init =============	
def Plugins(**kwargs):
	return [
	PluginDescriptor(
		name = _("TimeshiftPlus - Einstellungen"), 
		description = _("Anpassungen für VTi-Timeshift"),
		where = PluginDescriptor.WHERE_PLUGINMENU,
		icon = "TimeshiftPlus.png",
		needsRestart = True,
		fnc = main
	),
	PluginDescriptor(
		name = "TimeshiftPlus", 
		where = PluginDescriptor.WHERE_SESSIONSTART,
		fnc = sessionstart
	)
	]
